@@ -1,4 +1,5 @@
 use crate::opt::Opt;
+use crate::verbose_println;
 use anyhow::{Context, Result};
 use directories::{ProjectDirs, UserDirs};
 use std::env;
@@ -45,7 +46,14 @@ fn find_config_file(mut directory: PathBuf, recursive: bool) -> Result<Option<Co
 
 pub fn load_config(opt: &Opt) -> Result<Config> {
     match &opt.config_path {
-        Some(config_path) => read_config_file(config_path),
+        Some(config_path) => {
+            verbose_println!(
+                opt.verbose,
+                "config: explicit config path provided at {}",
+                config_path.display()
+            );
+            read_config_file(config_path)
+        }
         None => {
             let current_dir = match &opt.stdin_filepath {
                 Some(file_path) => file_path
@@ -54,13 +62,22 @@ pub fn load_config(opt: &Opt) -> Result<Config> {
                     .to_path_buf(),
                 None => env::current_dir().context("Could not find current directory")?,
             };
+            verbose_println!(
+                opt.verbose,
+                "config: starting config search from {} - recurisvely searching parents: {}",
+                current_dir.display(),
+                opt.search_parent_directories
+            );
             let config = find_config_file(current_dir, opt.search_parent_directories)?;
             match config {
                 Some(config) => Ok(config),
                 None => {
+                    verbose_println!(opt.verbose, "config: no configuration file found");
+
                     // Search the configuration directory for a file, if necessary
                     if opt.search_parent_directories {
                         // Look in `$HOME/.config/stylua`
+                        verbose_println!(opt.verbose, "config: looking in $HOME/.config/stylua");
                         if let Some(project_dirs) = ProjectDirs::from("", "", "stylua") {
                             if let Some(config) =
                                 find_config_file(project_dirs.config_dir().to_path_buf(), false)?
@@ -68,7 +85,9 @@ pub fn load_config(opt: &Opt) -> Result<Config> {
                                 return Ok(config);
                             }
                         }
+
                         // Look in `$HOME/.config`
+                        verbose_println!(opt.verbose, "config: looking in $HOME/.config");
                         if let Some(user_dirs) = UserDirs::new() {
                             if let Some(config) =
                                 find_config_file(user_dirs.home_dir().to_path_buf(), false)?
@@ -79,6 +98,7 @@ pub fn load_config(opt: &Opt) -> Result<Config> {
                     }
 
                     // Fallback to a default configuration
+                    verbose_println!(opt.verbose, "config: falling back to default config");
                     Ok(Config::default())
                 }
             }
