@@ -4,7 +4,6 @@ use crate::{
     formatters::{
         trivia::{FormatTriviaType, UpdateTrailingTrivia},
         trivia_util,
-        util::token_range,
     },
     shape::Shape,
     QuoteStyle,
@@ -33,15 +32,19 @@ pub enum EndTokenType {
     /// A closing brace at the end of a table.
     /// This means that the indent block that it was closing is formed from an indent range, rather than the current block indent level.
     ClosingBrace,
+    /// A closing parentheses at the end of e.g. a function call
+    /// This means that the indent block that it was closing is formed from an indent range, rather than the current block indent level.
+    ClosingParens,
 }
 
 #[macro_export]
 macro_rules! fmt_symbol {
-    ($ctx:expr, $token:expr, $x:expr) => {
+    ($ctx:expr, $token:expr, $x:expr, $shape:expr) => {
         crate::formatters::general::format_symbol(
             $ctx,
             $token,
             &TokenReference::symbol($x).unwrap(),
+            $shape,
         )
     };
 }
@@ -86,7 +89,7 @@ fn format_token<'ast>(
     ctx: &Context,
     token: &Token<'ast>,
     format_type: &FormatTokenType,
-    additional_indent_level: Option<usize>,
+    shape: Shape,
 ) -> (
     Token<'ast>,
     Option<Vec<Token<'ast>>>,
@@ -126,7 +129,7 @@ fn format_token<'ast>(
                 // Based off https://github.com/prettier/prettier/blob/181a325c1c07f1a4f3738665b7b28288dfb960bc/src/common/util.js#L439
                 lazy_static::lazy_static! {
                     static ref RE: regex::Regex = regex::Regex::new(r#"\\?(["'])|\\([\S\s])"#).unwrap();
-                    static ref UNNECESSARY_ESCAPES: regex::Regex = regex::Regex::new(r#"^[^\n\r"'0-7\\abfnrtuvxz]$"#).unwrap();
+                    static ref UNNECESSARY_ESCAPES: regex::Regex = regex::Regex::new(r#"^[^\n\r"'0-9\\abfnrtuvxz]$"#).unwrap();
                 }
                 let quote_to_use = get_quote_to_use(ctx, literal);
                 let literal = RE
@@ -185,14 +188,7 @@ fn format_token<'ast>(
 
             match format_type {
                 FormatTokenType::LeadingTrivia => {
-                    let additional_indent_level = additional_indent_level.unwrap_or(0)
-                        + ctx
-                            .get_range_indent_increase(token_range(token))
-                            .unwrap_or(0);
-                    leading_trivia = Some(vec![create_indent_trivia(
-                        ctx,
-                        Some(additional_indent_level),
-                    )]);
+                    leading_trivia = Some(vec![create_indent_trivia(ctx, shape)]);
                     trailing_trivia = Some(vec![create_newline_trivia(ctx)]);
                 }
                 FormatTokenType::TrailingTrivia => {
@@ -208,14 +204,7 @@ fn format_token<'ast>(
         }
         TokenType::MultiLineComment { blocks, comment } => {
             if let FormatTokenType::LeadingTrivia = format_type {
-                let additional_indent_level = additional_indent_level.unwrap_or(0)
-                    + ctx
-                        .get_range_indent_increase(token_range(token))
-                        .unwrap_or(0);
-                leading_trivia = Some(vec![create_indent_trivia(
-                    ctx,
-                    Some(additional_indent_level),
-                )]);
+                leading_trivia = Some(vec![create_indent_trivia(ctx, shape)]);
                 // Add a new line once the comment is completed
                 trailing_trivia = Some(vec![create_newline_trivia(ctx)]);
             }
@@ -242,7 +231,7 @@ fn load_token_trivia<'ast>(
     ctx: &Context,
     current_trivia: Vec<&Token<'ast>>,
     format_token_type: FormatTokenType,
-    additional_indent_level: Option<usize>,
+    shape: Shape,
 ) -> Vec<Token<'ast>> {
     let mut token_trivia = Vec::new();
 
@@ -250,21 +239,35 @@ fn load_token_trivia<'ast>(
     let mut trivia_iter = current_trivia.iter().peekable();
 
     while let Some(trivia) = trivia_iter.next() {
-        // Handle cases where the user has left a newline gap in between e.g. two statements
-        // If we are formatting trailing trivia, this can be ignored, as all trailing newlines will have already
-        // been handled by the formatter.
-        // If we are formatting leading trivia, we will allow a single newline to be kept in succession, if we
-        // find one.
         match trivia.token_type() {
             TokenType::Whitespace { characters } => {
-                if let FormatTokenType::LeadingTrivia = format_token_type {
-                    if characters.contains('\n') {
-                        newline_count_in_succession += 1;
-                        if newline_count_in_succession == 1 {
-                            // We have a case where we will allow a single newline to be kept
-                            token_trivia.push(create_newline_trivia(ctx));
+                // Handle cases where the user has left a newline gap in between e.g. two statements
+                // If we are formatting trailing trivia, this can be ignored, as all trailing newlines will have already
+                // been handled by the formatter.
+                // If we are formatting leading trivia, we will allow a single newline to be kept in succession, if we
+                // find one.
+                match format_token_type {
+                    FormatTokenType::LeadingTrivia => {
+                        if characters.contains('\n') {
+                            newline_count_in_succession += 1;
+                            if newline_count_in_succession == 1 {
+                                // We have a case where we will allow a single newline to be kept
+                                token_trivia.push(create_newline_trivia(ctx));
+                            }
                         }
                     }
+                    FormatTokenType::TrailingTrivia => {
+                        // If the next trivia is a MultiLineComment, and this whitespace is just spacing, then we
+                        // will preserve a single space
+                        if let Some(next_trivia) = trivia_iter.peek() {
+                            if let TokenType::MultiLineComment { .. } = next_trivia.token_type() {
+                                if !characters.contains('\n') {
+                                    token_trivia.push(Token::new(TokenType::spaces(1)))
+                                }
+                            }
+                        }
+                    }
+                    _ => (),
                 }
 
                 // Move to next trivia
@@ -292,12 +295,8 @@ fn load_token_trivia<'ast>(
             }
         }
 
-        let (token, leading_trivia, trailing_trivia) = format_token(
-            ctx,
-            trivia.to_owned(),
-            &format_token_type,
-            additional_indent_level,
-        );
+        let (token, leading_trivia, trailing_trivia) =
+            format_token(ctx, trivia.to_owned(), &format_token_type, shape);
         if let Some(mut trivia) = leading_trivia {
             token_trivia.append(&mut trivia);
         }
@@ -315,35 +314,27 @@ fn load_token_trivia<'ast>(
 pub fn format_token_reference<'a>(
     ctx: &Context,
     token_reference: &TokenReference<'a>,
+    shape: Shape,
 ) -> TokenReference<'a> {
     // Preserve comments in leading/trailing trivia
     let formatted_leading_trivia: Vec<Token<'a>> = load_token_trivia(
         ctx,
         token_reference.leading_trivia().collect(),
         FormatTokenType::LeadingTrivia,
-        None,
+        shape,
     );
     let formatted_trailing_trivia: Vec<Token<'a>> = load_token_trivia(
         ctx,
         token_reference.trailing_trivia().collect(),
         FormatTokenType::TrailingTrivia,
-        None,
+        shape,
     );
 
     let (token, _leading_trivia, _trailing_trivia) =
-        format_token(ctx, token_reference.token(), &FormatTokenType::Token, None);
+        format_token(ctx, token_reference.token(), &FormatTokenType::Token, shape);
 
     TokenReference::new(formatted_leading_trivia, token, formatted_trailing_trivia)
 }
-
-pub fn format_token_reference_mut<'ast>(
-    ctx: &mut Context,
-    token_reference: &TokenReference<'ast>,
-    _: Shape,
-) -> TokenReference<'ast> {
-    format_token_reference(ctx, token_reference)
-}
-
 // Formats a punctuation for a Punctuated sequence
 // Removes any trailing comments to be stored in a comments buffer
 pub fn format_punctuation<'ast>(
@@ -351,10 +342,7 @@ pub fn format_punctuation<'ast>(
 ) -> (TokenReference<'ast>, Vec<Token<'ast>>) {
     let trailing_comments = punctuation
         .trailing_trivia()
-        .filter(|token| {
-            token.token_kind() == TokenKind::SingleLineComment
-                || token.token_kind() == TokenKind::MultiLineComment
-        })
+        .filter(|x| trivia_util::trivia_is_comment(x))
         .map(|x| {
             // Prepend a single space beforehand
             vec![Token::new(TokenType::spaces(1)), x.to_owned()]
@@ -375,14 +363,14 @@ pub fn format_punctuation<'ast>(
 // Formats a Punctuated sequence with correct punctuated values
 // If there are any comments in between tied to the punctuation, they will be removed and stored in a returned comments buffer
 pub fn format_punctuated_buffer<'a, T, F>(
-    ctx: &mut Context,
+    ctx: &Context,
     old: &Punctuated<'a, T>,
     shape: Shape,
     value_formatter: F,
 ) -> (Punctuated<'a, T>, Vec<Token<'a>>)
 where
     T: std::fmt::Display,
-    F: Fn(&mut Context, &T, Shape) -> T,
+    F: Fn(&Context, &T, Shape) -> T,
 {
     let mut formatted: Punctuated<T> = Punctuated::new();
     let mut comments_buffer = Vec::new();
@@ -414,14 +402,14 @@ where
 /// This function assumes that there are no comments present which would lead to a syntax error if the list was collapsed.
 /// If not sure about comments, [`try_format_punctuated`] should be used instead.
 pub fn format_punctuated<'a, T, F>(
-    ctx: &mut Context,
+    ctx: &Context,
     old: &Punctuated<'a, T>,
     shape: Shape,
     value_formatter: F,
 ) -> Punctuated<'a, T>
 where
     T: std::fmt::Display,
-    F: Fn(&mut Context, &T, Shape) -> T,
+    F: Fn(&Context, &T, Shape) -> T,
 {
     let mut list: Punctuated<T> = Punctuated::new();
     let mut shape = shape;
@@ -430,7 +418,7 @@ where
         match pair {
             Pair::Punctuated(value, punctuation) => {
                 let value = value_formatter(ctx, value, shape);
-                let punctuation = fmt_symbol!(ctx, punctuation, ", ");
+                let punctuation = fmt_symbol!(ctx, punctuation, ", ", shape);
                 shape = shape + (value.to_string().len() + 2); // 2 = ", "
 
                 list.push(Pair::new(value, Some(punctuation)));
@@ -447,7 +435,7 @@ where
 
 // Formats a Punctuated sequence across multiple lines. Also indents each item by hang_level
 pub fn format_punctuated_multiline<'a, T, F>(
-    ctx: &mut Context,
+    ctx: &Context,
     old: &Punctuated<'a, T>,
     shape: Shape,
     value_formatter: F,
@@ -455,37 +443,33 @@ pub fn format_punctuated_multiline<'a, T, F>(
 ) -> Punctuated<'a, T>
 where
     T: Node<'a>,
-    F: Fn(&mut Context, &T, Shape) -> T,
+    F: Fn(&Context, &T, Shape) -> T,
 {
     let mut formatted: Punctuated<T> = Punctuated::new();
     let mut is_first = true; // Don't want to add an indent range for the first item, as it will be inline
-    let mut shape = shape;
 
     for pair in old.pairs() {
         // Indent the pair (unless its the first item)
-        if is_first {
+        let shape = if is_first {
             is_first = false;
+            shape
         } else {
-            let range = (
-                pair.start_position()
-                    .expect("no pair start position")
-                    .bytes(),
-                pair.end_position().expect("no pair end position").bytes(),
-            );
-            let additional_indent_level = ctx.get_range_indent_increase(range);
-            ctx.add_indent_range(range); // TODO: should this be before we get the range indent increaese?
-            shape = shape
-                .reset()
-                .with_additional_indent(additional_indent_level);
-        }
+            shape.reset()
+        };
+
+        // Include hang level if required
+        let shape = match hang_level {
+            Some(hang_level) => shape.with_indent(shape.indent().add_indent_level(hang_level)),
+            None => shape,
+        };
 
         match pair {
             Pair::Punctuated(value, punctuation) => {
                 let value = value_formatter(ctx, value, shape);
-                let punctuation = fmt_symbol!(ctx, punctuation, ",").update_trailing_trivia(
+                let punctuation = fmt_symbol!(ctx, punctuation, ",", shape).update_trailing_trivia(
                     FormatTriviaType::Append(vec![
                         create_newline_trivia(ctx),
-                        create_indent_trivia(ctx, hang_level),
+                        create_indent_trivia(ctx, shape),
                     ]),
                 );
                 formatted.push(Pair::new(value, Some(punctuation)));
@@ -503,14 +487,15 @@ where
 /// Formats a Punctuated sequence, depending on its layout. If the sequence contains comments, we will format
 /// across multiple lines
 pub fn try_format_punctuated<'a, T, F>(
-    ctx: &mut Context,
+    ctx: &Context,
     old: &Punctuated<'a, T>,
     shape: Shape,
     value_formatter: F,
+    hang_level: Option<usize>,
 ) -> Punctuated<'a, T>
 where
     T: Node<'a> + std::fmt::Display,
-    F: Fn(&mut Context, &T, Shape) -> T,
+    F: Fn(&Context, &T, Shape) -> T,
 {
     let mut format_multiline = false;
 
@@ -524,7 +509,7 @@ where
     }
 
     if format_multiline {
-        format_punctuated_multiline(ctx, old, shape, value_formatter, Some(1))
+        format_punctuated_multiline(ctx, old, shape, value_formatter, hang_level)
     } else {
         format_punctuated(ctx, old, shape, value_formatter)
     }
@@ -533,12 +518,13 @@ where
 pub fn format_contained_span<'ast>(
     ctx: &Context,
     contained_span: &ContainedSpan<'ast>,
+    shape: Shape,
 ) -> ContainedSpan<'ast> {
     let (start_token, end_token) = contained_span.tokens();
 
     ContainedSpan::new(
-        format_token_reference(ctx, start_token),
-        format_token_reference(ctx, end_token),
+        format_token_reference(ctx, start_token, shape),
+        format_token_reference(ctx, end_token, shape),
     )
 }
 
@@ -548,19 +534,20 @@ pub fn format_symbol<'ast>(
     ctx: &Context,
     current_symbol: &TokenReference<'ast>,
     wanted_symbol: &TokenReference<'ast>,
+    shape: Shape,
 ) -> TokenReference<'ast> {
     // Preserve comments in leading/trailing trivia
     let mut formatted_leading_trivia: Vec<Token<'ast>> = load_token_trivia(
         ctx,
         current_symbol.leading_trivia().collect(),
         FormatTokenType::LeadingTrivia,
-        None,
+        shape,
     );
     let mut formatted_trailing_trivia: Vec<Token<'ast>> = load_token_trivia(
         ctx,
         current_symbol.trailing_trivia().collect(),
         FormatTokenType::TrailingTrivia,
-        None,
+        shape,
     );
 
     // Add on any whitespace created in the new symbol
@@ -590,26 +577,23 @@ pub fn format_symbol<'ast>(
 pub fn format_end_token<'ast>(
     ctx: &Context,
     current_token: &TokenReference<'ast>,
-    token_type: EndTokenType,
+    _token_type: EndTokenType,
+    shape: Shape,
 ) -> TokenReference<'ast> {
     // Indent any comments leading a token, as these comments are technically part of the function body block
     let formatted_leading_trivia: Vec<Token<'ast>> = load_token_trivia(
         ctx,
         current_token.leading_trivia().collect(),
         FormatTokenType::LeadingTrivia,
-        match token_type {
-            // The indent level we are currently at is one less (as we are at the end token, not the indented block).
-            // The comment is present inside the indented block
-            EndTokenType::BlockEnd => Some(1),
-            // We are closing an "indent range". The leading comments will still be inside this range, so we don't need an extra indent level
-            EndTokenType::ClosingBrace => None,
-        },
+        // The indent level we are currently at is one less (as we are at the block closing token, not the indented block).
+        // The comment is present inside the indented block
+        shape.increment_additional_indent(),
     );
     let formatted_trailing_trivia: Vec<Token<'ast>> = load_token_trivia(
         ctx,
         current_token.trailing_trivia().collect(),
         FormatTokenType::TrailingTrivia,
-        None,
+        shape,
     );
 
     // Special case for block end tokens:
@@ -664,17 +648,19 @@ fn pop_until_no_whitespace(trivia: &mut Vec<Token>) {
 /// Format the EOF token.
 /// This is done by removing any leading whitespace, whilst preserving leading comments.
 /// An EOF token has no trailing trivia
-pub fn format_eof<'ast>(ctx: &mut Context, eof: &TokenReference<'ast>) -> TokenReference<'ast> {
+pub fn format_eof<'ast>(
+    ctx: &Context,
+    eof: &TokenReference<'ast>,
+    shape: Shape,
+) -> TokenReference<'ast> {
     check_should_format!(ctx, eof);
 
     // Need to preserve any comments in leading_trivia if present
-    // The indent level will be 0 at this point, as we have finished the whole file, so we need to one-index it again
-    ctx.increment_indent_level();
     let mut formatted_leading_trivia: Vec<Token<'ast>> = load_token_trivia(
         ctx,
         eof.leading_trivia().collect(),
         FormatTokenType::LeadingTrivia,
-        None,
+        shape,
     );
 
     let only_whitespace = formatted_leading_trivia
